@@ -14,7 +14,7 @@ from urllib3.exceptions import IncompleteRead
 
 import fsspec  # noqa: F401
 from fsspec.spec import AbstractBufferedFile
-from fsspec.utils import infer_storage_options, tokenize, setup_logging as setup_logger
+from fsspec.utils import tokenize, setup_logging as setup_logger
 from fsspec.asyn import (
     AsyncFileSystem,
     AbstractAsyncStreamedFile,
@@ -56,7 +56,7 @@ if "S3FS_LOGGING_LEVEL" in os.environ:
     setup_logging()
 
 
-MANAGED_COPY_THRESHOLD = 5 * 2**30
+MANAGED_COPY_THRESHOLD = 150 * 2**20
 # Certain rate-limiting responses can send invalid XML
 # (see https://github.com/fsspec/s3fs/issues/484), which can result in a parser error
 # deep within botocore. So we treat those as retryable as well, even though there could
@@ -391,8 +391,9 @@ class S3FileSystem(AsyncFileSystem):
         Assume that we want to use version_aware mode for
         the filesystem.
         """
-        url_storage_opts = infer_storage_options(urlpath)
-        url_query = url_storage_opts.get("url_query")
+        from urllib.parse import urlsplit
+
+        url_query = urlsplit(urlpath).query
         out = {}
         if url_query is not None:
             from urllib.parse import parse_qs
@@ -641,6 +642,10 @@ class S3FileSystem(AsyncFileSystem):
         mode: string
             One of 'r', 'w', 'a', 'rb', 'wb', or 'ab'. These have the same meaning
             as they do for the built-in `open` function.
+            "x" mode, exclusive write, is only known to work on AWS S3, and
+            requires botocore>1.35.20. If the file is multi-part (i.e., has more
+            than one block), the condition is only checked on commit; if this fails,
+            the MPU is aborted.
         block_size: int
             Size of data-node blocks if reading
         fill_cache: bool
@@ -852,7 +857,7 @@ class S3FileSystem(AsyncFileSystem):
         #         return super().find(path)
         #     # else: we refresh anyway, having at least two missing trees
         out = await self._lsdir(path, delimiter="", prefix=prefix, **kwargs)
-        if not out and key:
+        if (not out and key) and not prefix:
             try:
                 out = [await self._info(path)]
             except FileNotFoundError:
@@ -1134,15 +1139,30 @@ class S3FileSystem(AsyncFileSystem):
         return await _error_wrapper(_call_and_read, retries=self.retries)
 
     async def _pipe_file(
-        self, path, data, chunksize=50 * 2**20, max_concurrency=None, **kwargs
+        self,
+        path,
+        data,
+        chunksize=50 * 2**20,
+        max_concurrency=None,
+        mode="overwrite",
+        **kwargs,
     ):
+        """
+        mode=="create", exclusive write, is only known to work on AWS S3, and
+        requires botocore>1.35.20
+        """
         bucket, key, _ = self.split_path(path)
         concurrency = max_concurrency or self.max_concurrency
         size = len(data)
+        if mode == "create":
+            match = {"IfNoneMatch": "*"}
+        else:
+            match = {}
+
         # 5 GB is the limit for an S3 PUT
         if size < min(5 * 2**30, 2 * chunksize):
             out = await self._call_s3(
-                "put_object", Bucket=bucket, Key=key, Body=data, **kwargs
+                "put_object", Bucket=bucket, Key=key, Body=data, **kwargs, **match
             )
             self.invalidate_cache(path)
             return out
@@ -1154,32 +1174,37 @@ class S3FileSystem(AsyncFileSystem):
             ranges = list(range(0, len(data), chunksize))
             inds = list(range(0, len(ranges), concurrency)) + [len(ranges)]
             parts = []
-            for start, stop in zip(inds[:-1], inds[1:]):
-                out = await asyncio.gather(
-                    *[
-                        self._call_s3(
-                            "upload_part",
-                            Bucket=bucket,
-                            PartNumber=i + 1,
-                            UploadId=mpu["UploadId"],
-                            Body=data[ranges[i] : ranges[i] + chunksize],
-                            Key=key,
-                        )
-                        for i in range(start, stop)
-                    ]
+            try:
+                for start, stop in zip(inds[:-1], inds[1:]):
+                    out = await asyncio.gather(
+                        *[
+                            self._call_s3(
+                                "upload_part",
+                                Bucket=bucket,
+                                PartNumber=i + 1,
+                                UploadId=mpu["UploadId"],
+                                Body=data[ranges[i] : ranges[i] + chunksize],
+                                Key=key,
+                            )
+                            for i in range(start, stop)
+                        ]
+                    )
+                    parts.extend(
+                        {"PartNumber": i + 1, "ETag": o["ETag"]}
+                        for i, o in zip(range(start, stop), out)
+                    )
+                await self._call_s3(
+                    "complete_multipart_upload",
+                    Bucket=bucket,
+                    Key=key,
+                    UploadId=mpu["UploadId"],
+                    MultipartUpload={"Parts": parts},
+                    **match,
                 )
-                parts.extend(
-                    {"PartNumber": i + 1, "ETag": o["ETag"]}
-                    for i, o in zip(range(start, stop), out)
-                )
-            await self._call_s3(
-                "complete_multipart_upload",
-                Bucket=bucket,
-                Key=key,
-                UploadId=mpu["UploadId"],
-                MultipartUpload={"Parts": parts},
-            )
-            self.invalidate_cache(path)
+                self.invalidate_cache(path)
+            except Exception:
+                await self._abort_mpu(bucket, key, mpu["UploadId"])
+                raise
 
     async def _put_file(
         self,
@@ -1188,8 +1213,13 @@ class S3FileSystem(AsyncFileSystem):
         callback=_DEFAULT_CALLBACK,
         chunksize=50 * 2**20,
         max_concurrency=None,
+        mode="overwrite",
         **kwargs,
     ):
+        """
+        mode=="create", exclusive write, is only known to work on AWS S3, and
+        requires botocore>1.35.20
+        """
         bucket, key, _ = self.split_path(rpath)
         if os.path.isdir(lpath):
             if key:
@@ -1199,6 +1229,10 @@ class S3FileSystem(AsyncFileSystem):
                 await self._mkdir(lpath)
         size = os.path.getsize(lpath)
         callback.set_size(size)
+        if mode == "create":
+            match = {"IfNoneMatch": "*"}
+        else:
+            match = {}
 
         if "ContentType" not in kwargs:
             content_type, _ = mimetypes.guess_type(lpath)
@@ -1209,7 +1243,7 @@ class S3FileSystem(AsyncFileSystem):
             if size < min(5 * 2**30, 2 * chunksize):
                 chunk = f0.read()
                 await self._call_s3(
-                    "put_object", Bucket=bucket, Key=key, Body=chunk, **kwargs
+                    "put_object", Bucket=bucket, Key=key, Body=chunk, **kwargs, **match
                 )
                 callback.relative_update(size)
             else:
@@ -1217,25 +1251,31 @@ class S3FileSystem(AsyncFileSystem):
                 mpu = await self._call_s3(
                     "create_multipart_upload", Bucket=bucket, Key=key, **kwargs
                 )
-                out = await self._upload_file_part_concurrent(
-                    bucket,
-                    key,
-                    mpu,
-                    f0,
-                    callback=callback,
-                    chunksize=chunksize,
-                    max_concurrency=max_concurrency,
-                )
-                parts = [
-                    {"PartNumber": i + 1, "ETag": o["ETag"]} for i, o in enumerate(out)
-                ]
-                await self._call_s3(
-                    "complete_multipart_upload",
-                    Bucket=bucket,
-                    Key=key,
-                    UploadId=mpu["UploadId"],
-                    MultipartUpload={"Parts": parts},
-                )
+                try:
+                    out = await self._upload_file_part_concurrent(
+                        bucket,
+                        key,
+                        mpu,
+                        f0,
+                        callback=callback,
+                        chunksize=chunksize,
+                        max_concurrency=max_concurrency,
+                    )
+                    parts = [
+                        {"PartNumber": i + 1, "ETag": o["ETag"]}
+                        for i, o in enumerate(out)
+                    ]
+                    await self._call_s3(
+                        "complete_multipart_upload",
+                        Bucket=bucket,
+                        Key=key,
+                        UploadId=mpu["UploadId"],
+                        MultipartUpload={"Parts": parts},
+                        **match,
+                    )
+                except Exception:
+                    await self._abort_mpu(bucket, key, mpu["UploadId"])
+                    raise
         while rpath:
             self.invalidate_cache(rpath)
             rpath = self._parent(rpath)
@@ -1275,17 +1315,14 @@ class S3FileSystem(AsyncFileSystem):
                     chunks.append(chunk)
             if not chunks:
                 break
-            if len(chunks) > 1:
-                out.extend(
-                    await asyncio.gather(
-                        *[
-                            _upload_chunk(chunk, len(out) + i)
-                            for i, chunk in enumerate(chunks, 1)
-                        ]
-                    )
+            out.extend(
+                await asyncio.gather(
+                    *[
+                        _upload_chunk(chunk, len(out) + i)
+                        for i, chunk in enumerate(chunks, 1)
+                    ]
                 )
-            else:
-                out.append(await _upload_chunk(chunk, len(out) + 1))
+            )
         return out
 
     async def _get_file(
@@ -1862,7 +1899,7 @@ class S3FileSystem(AsyncFileSystem):
         )
         self.invalidate_cache(path2)
 
-    async def _copy_managed(self, path1, path2, size, block=5 * 2**30, **kwargs):
+    async def _copy_managed(self, path1, path2, size, block=50 * 2**20, **kwargs):
         """Copy file between locations on S3 as multi-part
 
         block: int
@@ -1884,7 +1921,7 @@ class S3FileSystem(AsyncFileSystem):
                 Key=key,
                 PartNumber=i + 1,
                 UploadId=mpu["UploadId"],
-                CopySource=path1,
+                CopySource=self._strip_protocol(path1),
                 CopySourceRange="bytes=%i-%i" % (brange_first, brange_last),
             )
             for i, (brange_first, brange_last) in enumerate(_get_brange(size, block))
@@ -1941,18 +1978,22 @@ class S3FileSystem(AsyncFileSystem):
 
     list_multipart_uploads = sync_wrapper(_list_multipart_uploads)
 
+    async def _abort_mpu(self, bucket, key, mpu):
+        await self._call_s3(
+            "abort_multipart_upload",
+            Bucket=bucket,
+            Key=key,
+            UploadId=mpu,
+        )
+
+    abort_mpu = sync_wrapper(_abort_mpu)
+
     async def _clear_multipart_uploads(self, bucket):
         """Remove any partial uploads in the bucket"""
-        out = await self._list_multipart_uploads(bucket)
         await asyncio.gather(
             *[
-                self._call_s3(
-                    "abort_multipart_upload",
-                    Bucket=bucket,
-                    Key=upload["Key"],
-                    UploadId=upload["UploadId"],
-                )
-                for upload in out
+                self._abort_mpu(bucket, upload["Key"], upload["UploadId"])
+                for upload in await self._list_multipart_uploads(bucket)
             ]
         )
 
@@ -2354,7 +2395,9 @@ class S3File(AbstractBufferedFile):
                 if len(part_data) == 0:
                     return
                 part = len(self.parts) + 1
-                logger.debug("Upload chunk %s, %s" % (self, part))
+                logger.debug(
+                    "Upload chunk %s, %s; %s bytes" % (self, part, len(part_data))
+                )
 
                 out = self._call_s3(
                     "upload_part",
@@ -2414,6 +2457,10 @@ class S3File(AbstractBufferedFile):
                 raise RuntimeError
         else:
             logger.debug("Complete multi-part upload for %s " % self)
+            if "x" in self.mode:
+                match = {"IfNoneMatch": "*"}
+            else:
+                match = {}
             part_info = {"Parts": self.parts}
             write_result = self._call_s3(
                 "complete_multipart_upload",
@@ -2421,6 +2468,7 @@ class S3File(AbstractBufferedFile):
                 Key=self.key,
                 UploadId=self.mpu["UploadId"],
                 MultipartUpload=part_info,
+                **match,
             )
 
         if self.fs.version_aware:
@@ -2443,12 +2491,7 @@ class S3File(AbstractBufferedFile):
 
     def _abort_mpu(self):
         if self.mpu:
-            self._call_s3(
-                "abort_multipart_upload",
-                Bucket=self.bucket,
-                Key=self.key,
-                UploadId=self.mpu["UploadId"],
-            )
+            self.fs.abort_mpu(self.bucket, self.key, self.mpu["UploadId"])
             self.mpu = None
 
 
