@@ -79,6 +79,7 @@ def s3_base():
         os.environ["AWS_SECRET_ACCESS_KEY"] = "foo"
     if "AWS_ACCESS_KEY_ID" not in os.environ:
         os.environ["AWS_ACCESS_KEY_ID"] = "foo"
+    os.environ.pop("AWS_PROFILE", None)
 
     print("server up")
     yield
@@ -1017,6 +1018,24 @@ def test_put_file_with_callback(s3, tmpdir, size):
     assert cb.size == os.stat(test_file).st_size
     assert cb.value == cb.size
 
+    assert s3.size(test_bucket_name + "/temp") == 11 * size
+
+
+@pytest.mark.parametrize("factor", [1, 5, 6])
+def test_put_file_does_not_truncate(s3, tmpdir, factor):
+    test_file = str(tmpdir.join("test.json"))
+
+    chunksize = 5 * 2**20
+    block = b"x" * chunksize
+
+    with open(test_file, "wb") as f:
+        f.write(block * factor)
+
+    s3.put_file(
+        test_file, test_bucket_name + "/temp", max_concurrency=5, chunksize=chunksize
+    )
+    assert s3.size(test_bucket_name + "/temp") == factor * chunksize
+
 
 @pytest.mark.parametrize("size", [2**10, 2**20, 10 * 2**20])
 def test_pipe_cat_big(s3, size):
@@ -1216,15 +1235,15 @@ def test_write_limit(s3):
 
 
 def test_write_small_secure(s3):
-    # Unfortunately moto does not yet support enforcing SSE policies.  It also
-    # does not return the correct objects that can be used to test the results
-    # effectively.
-    # This test is left as a placeholder in case moto eventually supports this.
-    sse_params = SSEParams(server_side_encryption="aws:kms")
-    with s3.open(secure_bucket_name + "/test", "wb", writer_kwargs=sse_params) as f:
+    s3 = S3FileSystem(
+        s3_additional_kwargs={"ServerSideEncryption": "aws:kms"},
+        client_kwargs={"endpoint_url": endpoint_uri},
+    )
+    s3.mkdir("mybucket")
+    with s3.open("mybucket/test", "wb") as f:
         f.write(b"hello")
-    assert s3.cat(secure_bucket_name + "/test") == b"hello"
-    sync(s3.loop, s3.s3.head_object, Bucket=secure_bucket_name, Key="test")
+    assert s3.cat("mybucket/test") == b"hello"
+    sync(s3.loop, s3.s3.head_object, Bucket="mybucket", Key="test")
 
 
 def test_write_large_secure(s3):
@@ -1268,10 +1287,10 @@ def test_write_blocks(s3):
         assert f.mpu
         assert f.parts
     assert s3.info(test_bucket_name + "/temp")["size"] == 6 * 2**20
-    with s3.open(test_bucket_name + "/temp", "wb", block_size=10 * 2**20) as f:
+    with s3.open(test_bucket_name + "/temp2", "wb", block_size=10 * 2**20) as f:
         f.write(b"a" * 15 * 2**20)
         assert f.buffer.tell() == 0
-    assert s3.info(test_bucket_name + "/temp")["size"] == 15 * 2**20
+    assert s3.info(test_bucket_name + "/temp2")["size"] == 15 * 2**20
 
 
 def test_readline(s3):
@@ -1387,7 +1406,8 @@ def test_append(s3):
         f.write(b"a" * 10 * 2**20)
     with s3.open(a, "ab") as f:
         pass  # append, no write, big file
-    assert s3.cat(a) == b"a" * 10 * 2**20
+    data = s3.cat(a)
+    assert len(data) == 10 * 2**20 and set(data) == set(b"a")
 
     with s3.open(a, "ab") as f:
         assert f.parts is None
@@ -1395,13 +1415,18 @@ def test_append(s3):
         assert f.parts
         assert f.tell() == 10 * 2**20
         f.write(b"extra")  # append, small write, big file
-    assert s3.cat(a) == b"a" * 10 * 2**20 + b"extra"
+    data = s3.cat(a)
+    assert len(data) == 10 * 2**20 + len(b"extra")
+    assert data[-5:] == b"extra"
 
     with s3.open(a, "ab") as f:
         assert f.tell() == 10 * 2**20 + 5
         f.write(b"b" * 10 * 2**20)  # append, big write, big file
         assert f.tell() == 20 * 2**20 + 5
-    assert s3.cat(a) == b"a" * 10 * 2**20 + b"extra" + b"b" * 10 * 2**20
+    data = s3.cat(a)
+    assert len(data) == 10 * 2**20 + len(b"extra") + 10 * 2**20
+    assert data[10 * 2**20 : 10 * 2**20 + 5] == b"extra"
+    assert set(data[-10 * 2**20 :]) == set(b"b")
 
     # Keep Head Metadata
     head = dict(
@@ -2144,6 +2169,42 @@ def test_via_fsspec(s3):
         assert f.read() == b"hello"
 
 
+@pytest.mark.parametrize(
+    ["raw_url", "expected_url", "expected_version_aware"],
+    [
+        (
+            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/abc/123.jpg",
+            "arn:aws:s3:us-west-2:123456789012:accesspoint/abc/123.jpg",
+            False,
+        ),
+        (
+            "s3://arn:aws:s3:us-west-2:123456789012:accesspoint/abc/123.jpg?versionId=some_version_id",
+            "arn:aws:s3:us-west-2:123456789012:accesspoint/abc/123.jpg?versionId=some_version_id",
+            True,
+        ),
+        (
+            "s3://xyz/abc/123.jpg",
+            "xyz/abc/123.jpg",
+            False,
+        ),
+        (
+            "s3://xyz/abc/123.jpg?versionId=some_version_id",
+            "xyz/abc/123.jpg?versionId=some_version_id",
+            True,
+        ),
+    ],
+)
+def test_fsspec_url_to_fs_compatability(
+    s3, raw_url, expected_url, expected_version_aware
+):
+    import fsspec
+
+    fs, url = fsspec.url_to_fs(raw_url)
+    assert isinstance(fs, type(s3))
+    assert fs.version_aware is expected_version_aware
+    assert url == expected_url
+
+
 def test_repeat_exists(s3):
     fn = "s3://" + test_bucket_name + "/file1"
     s3.touch(fn)
@@ -2480,6 +2541,7 @@ def test_find_with_prefix(s3):
     s3.touch(test_bucket_name + "/prefixes2")
     assert len(s3.find(test_bucket_name + "/prefixes")) == 100
     assert len(s3.find(test_bucket_name, prefix="prefixes")) == 101
+    assert len(s3.find(test_bucket_name + "/prefixes", prefix="test2_")) == 0
 
     assert len(s3.find(test_bucket_name + "/prefixes/test_")) == 0
     assert len(s3.find(test_bucket_name + "/prefixes", prefix="test_")) == 100
@@ -2812,3 +2874,30 @@ async def test_invalidate_cache(s3: s3fs.S3FileSystem) -> None:
     await s3._pipe_file(f"{test_bucket_name}/a/c.txt", data=b"abc")
     after = await s3._ls(f"{test_bucket_name}/a/")
     assert sorted(after) == ["test/a/b.txt", "test/a/c.txt"]
+
+
+@pytest.mark.xfail(reason="moto doesn't support conditional MPU")
+def test_pipe_exclusive_big(s3):
+    chunksize = 5 * 2**20  # minimum allowed
+    data = b"x" * chunksize * 3
+    s3.pipe(f"{test_bucket_name}/afile", data, mode="overwrite", chunksize=chunksize)
+    s3.pipe(f"{test_bucket_name}/afile", data, mode="overwrite", chunksize=chunksize)
+    with pytest.raises(FileExistsError):
+        s3.pipe(f"{test_bucket_name}/afile", data, mode="create", chunksize=chunksize)
+    assert not s3.list_multipart_uploads(test_bucket_name)
+
+
+@pytest.mark.xfail(reason="moto doesn't support conditional MPU")
+def test_put_exclusive_big(s3, tempdir):
+    chunksize = 5 * 2**20  # minimum allowed
+    data = b"x" * chunksize * 3
+    fn = f"{tempdir}/afile"
+    with open(fn, "wb") as f:
+        f.write(fn)
+    s3.put(fn, f"{test_bucket_name}/afile", data, mode="overwrite", chunksize=chunksize)
+    s3.put(fn, f"{test_bucket_name}/afile", data, mode="overwrite", chunksize=chunksize)
+    with pytest.raises(FileExistsError):
+        s3.put(
+            fn, f"{test_bucket_name}/afile", data, mode="create", chunksize=chunksize
+        )
+    assert not s3.list_multipart_uploads(test_bucket_name)
